/* XVCL - the xview classes

   XVCL might one day be a hopefully useful class library
   for xview programming in C++. Just now, it slowly grows
   as I write programs, so many many things are missing.

   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file declares generic classes for windows, buttons and so on. 

   98/11/30, Frank Rustemeyer <mambres@mud.tap.de>: Released.
*/

#ifndef _XVCLH
#define _XVCLH
#include <X11/X.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>
#include <xview/scrollbar.h>
#include <xview/notify.h>

#define END NULL
#define FILE_OPEN 1
#define FILE_SAVE 2

class xobj // any Xview object (abstract base class)
{   public:
        Xv_object get_handle() { return handle; }
        void set_handle(Xv_object h) { handle = h; }
        void show(int i);
        int shown(void);
 
        virtual void destroy(void) {};
    private: 
        Xv_object handle;
    };

class window : public xobj   // a window (abstract base class)
{   public:
        void busy(int b) { xv_set(get_handle(), FRAME_BUSY, b, END); }
        void create_frame(int x, int y, char *label);
        void set_min_size(int x, int y);
        void set_max_size(int x, int y);
        void notify_destroy(void);

        virtual void paint(Canvas c, Xv_Window x, Display *d, 
                           Window w, Xv_xrectlist *xr) {};
        virtual void resize(int x, int y) {};
        virtual ~window() {};
        virtual void button_pressed(Xv_object item) {}; 
                      // react on buttons pressed 
        virtual void checkbox_selected(Xv_object item, int value) {};
        virtual void menu_selected(char *item) {};
        virtual void file_open(char *path) {};
        virtual void file_save(char *path) {};
        virtual void event(Xv_Window wnd, Event *e,  
                           Notify_arg arg, Notify_event_type t) {}; 
                      // event handler
        virtual Notify_value input_pending(int fd) {};
    };

class dialog : public window      // dialog box (abstract base class)
{   public:
        dialog();
	~dialog();
        virtual void destroy(void); //process a destroy message
	int run(void);
	  // run the dialog. can return success or failure, if useful
        void enddialog(int returnval);
    protected:
        int retval; // return value;
    private:
    };

class panel : public xobj
{   public:
        panel(class window *w);
        set_height(int h);
        set_width(int w);
        void button_pressed(Xv_object item);
        void checkbox_selected(Xv_object item, int value);
        void menu_selected(char *item);
        Display * query_display(void);
    private:
        window *owner;
    };

class canvas : public xobj
{   public:
        canvas(class window *w, int x, int y, int ncols, Xv_singlecolor *cols);
        canvas(class window *w, int x, int y, Cms cms);
        set_fixed_size(int x, int y);
        add_scrollbars(void);
        void scrollto(int x, int y);
        void centre(int x, int y);
        int visible(int x, int y);
        void mouse_events(void);
        void keyboard_events(void);
        Xv_object paint_window(void);
        Display * query_display(void);
        unsigned long colour(int x) { return coltable[x]; };
        void invalidate(void);
    private:
        class window *owner;
        Cms cms;
        unsigned long *coltable;
        Scrollbar s_hor;
        Scrollbar s_ver;
    };

class panelitem : public xobj
{   public:
       ~panelitem(void);
       void set_pos(int x, int y) { xv_set(get_handle(), XV_X, x, XV_Y, y, NULL); }
    protected:
       class panel *owner;
    };

class button : public panelitem // button for special commands
{   public:
	button(panel *p, char * text);
        void react(void);
        void add_menu(class menu *m);
        void enable(int state);
    };

class gauge : public panelitem
{   public:
        gauge(panel *p);
        void set_value(int v);
        void set_width(int w);
        void set_range(int min, int max);
    };

class checkbox : public panelitem 
{   public:
        checkbox(panel *p, char *text, int val);
        void selected(int val);
        int query_value(void);
        void set_value(int val);
    };

class textbox : public panelitem 
{   public:
        textbox(panel *p, char *text, char *val, int maxlen);
        char *query_value(void);
        void set_value(char *text);
    };

class numeric : public panelitem
{   public:
        numeric(panel *p, char *text, int val, int min, int max);
        int query_value(void);
        void set_value(int val);
    };

class textview : public panelitem
{   public:
        textview(panel *p, char *text);
        void set_value(char *text);
    };

class imageview : public panelitem
{   public:
        imageview(panel *p, char **img);
    };

class listbox : public panelitem
{   public:
        listbox(panel *p, char *title, int height);
        void add_string(char *s);
        int get_value(void);
        void set_value(int n);
    }; 

class menu : public panelitem
{   public:
        menu(window *owner, int size, char **entries);
        ~menu(void);
        void selected(Menu_item item);
    private:
        class window *owner;
    };
       
class messagebox : public xobj
{   public:
        messagebox(xobj *owner, char *text);
    };

class yesnobox : public xobj
{   public:
        yesnobox(xobj *owner, char *text);
        int ok(void);
    private:
        int val;
    };

class infobox: public window
{   public:
        infobox(xobj *owner, char *text);
    };

class filechooser : public xobj
{   public:
        filechooser(window *owner, int type, char *filter);
        void activate(void);
        int file_name(char *path, char *file);
        int file_save(char *path);
    private:
        window *owner;
    };

// *********************** The distributor object *****************************

//prototypes for functions that distribute windows messages

void centralpaint(Canvas c, Xv_Window x, Display *d, Window w, Xv_xrectlist *xr);
void centralresize(Canvas c, int x, int y);
void centraldestroy(Frame f);
void centralbuttonpressed(Panel_item item);
void centralevent(Xv_window w, Event *e, Notify_arg arg, Notify_event_type t);
void centralselected(Panel_item item, int val);
void centralmenu(Menu menu, Menu_item item);
int centralfileopen(Xv_object File_chooser, char *path, char *file);
int centralfilesave(Xv_object File_chooser, char *path);
Notify_value centralinput(Notify_client c, int fd);

class distributor
  // this object is responsible for distributing the windows messages
  // to the window-objects. It holds a list of matching pairs of a
  // window object pointer and a window handle. When a message is
  // received, the window object pointer is retrieved from the list
  // and the virtual wndproc in this object is called.
{   public:
	distributor(void);
	~distributor(void);
	xobj * findobject(Xv_object h);
	  // find window object belonging to handle hW
	int addwindow(xobj *w, Xv_object h);
	  // register new window with object
	int removewindow(Xv_object h);       // remove object and window
    private:
	xobj * creator;  // store an object until its window is created
	int nwindows;      // number of current windows
	xobj **warray;   // array of window objects
	Xv_object *harray;      // array of window handles
    };

#endif

